//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <vcl/jpeg.hpp>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "Main.h"
#include "Options.h"
#include "SafeRegistry.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "ThemeMgr"
#pragma resource "*.dfm"

TMainWnd *MainWnd;
//---------------------------------------------------------------------------
__fastcall TMainWnd::TMainWnd(TComponent* Owner)
   : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::FormCreate(TObject *Sender)
{
   if (DirectDrawCreate(NULL, &dd, NULL) != DD_OK)
   {
      Application->MessageBox("This program requires DirectX 3 or higher.",
                              "Random Background", MB_OK | MB_ICONERROR);
      exit(1);
   }
   dd->QueryInterface(IID_IDirectDraw4, (void **) &lpDD);
   dd->Release();
   lpDD->SetCooperativeLevel(NULL, DDSCL_NORMAL);
   IconData.cbSize = sizeof(NOTIFYICONDATA);
   IconData.hWnd = Handle;
   IconData.uID = 1;
   IconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
   IconData.uCallbackMessage = WM_ICONMESSAGE;
   IconData.hIcon = Application->Icon->Handle;
   strcpy(IconData.szTip, "Random Background - Loading");
   Shell_NotifyIcon(NIM_ADD, &IconData);
   srand((unsigned) time(0));
   Timer->Enabled = true;
   TimeLeft = 1;
   BackgroundFile = "";
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::TMExitClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::TMOptionsClick(TObject *Sender)
{
   if (!OptionsWnd->Visible)
   {
      MainWnd->TMOptions->Enabled = false;
      OptionsWnd->ShowModal();
      MainWnd->TMOptions->Enabled = true;
   }
   else
      SetForegroundWindow(OptionsWnd->Handle);
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::IconMessage(TMessage& Msg)
{
   POINT p;

   if (Msg.LParam == WM_RBUTTONUP)
   {
      GetCursorPos(&p);
      SetForegroundWindow(Handle);
      TrayMenu->Popup(p.x, p.y);
      PostMessage(Handle, WM_NULL, 0, 0);
   }
   else if (Msg.LParam == WM_LBUTTONUP)
      TMOptionsClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::FormClose(TObject *Sender, TCloseAction &Action)
{
   Shell_NotifyIcon(NIM_DELETE, &IconData);
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::TimerTimer(TObject *Sender)
{
   AnsiString msg;

   TimeLeft--;
   msg.printf("%s - %i:%02i", BackgroundFile.c_str(),
              TimeLeft / 60, TimeLeft % 60);
   if (msg.Length() > 60)
      msg = "..." + msg.SubString((msg.Length() - 60) + 1, 60);
   lstrcpy(IconData.szTip, msg.c_str());
   Shell_NotifyIcon(NIM_MODIFY, &IconData);
   msg.printf("Time remaining: %i:%02i", TimeLeft / 60, TimeLeft % 60);
   OptionsWnd->Remaining->Caption = msg;
   if (TimeLeft <= 0)
   {
      TimeLeft = OptionsWnd->TimeInterval * 60;
      if (lpDD->TestCooperativeLevel() != DD_OK)
         return;
      if (GetSystemMetrics(SM_REMOTESESSION) != 0)
         return;
      lstrcpy(IconData.szTip, "Random Background - (changing)");
      Shell_NotifyIcon(NIM_MODIFY, &IconData);
      OptionsWnd->Remaining->Caption = "Time remaining: (changing)";
      OptionsWnd->Remaining->Refresh();
      SetPriorityClass(GetCurrentProcess(), IDLE_PRIORITY_CLASS);
      RandomizeBackground();
      SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
   }
   else if ((OptionsWnd->TimeInterval * 60) < TimeLeft)
      TimeLeft = OptionsWnd->TimeInterval * 60;
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::TMResetTimerClick(TObject *Sender)
{
   TimeLeft = OptionsWnd->TimeInterval * 60;
}
//---------------------------------------------------------------------------
void TMainWnd::RandomizeBackground()
{
   TStringList *names;
   AnsiString path;

   names = new TStringList();
   names->Sorted = true;
   LoadDirectory(OptionsWnd->BackgroundPath, names);
   if (names->Count > 0)
   {
      if (OptionsWnd->MultiMon)
         DoMultiMon(names);
      else
      {
         ResetTile();
         path = PickWallpaper(names);
         SetWallpaper(path.c_str());
      }
   }
   delete names;
}
//---------------------------------------------------------------------------
void TMainWnd::LoadDirectory(AnsiString path, TStrings *names)
{
   AnsiString base;
   TSearchRec sr;
   int bl;

   bl = OptionsWnd->BackgroundPath.Length();
   if ((path.Length() - bl) > 0)
      base = path.SubString(bl + 2, (path.Length() - bl) - 1) + "\\";
   if (OptionsWnd->RecurseDirs)
      if (FindFirst(path + "\\*", faDirectory, sr) == 0)
      {
         do
            if ((sr.Name != ".") && (sr.Name != "..") &&
                ((sr.Attr & faDirectory) == faDirectory))
               LoadDirectory(path + "\\" + sr.Name, names);
         while (FindNext(sr) == 0);
         FindClose(sr);
      }
   if (OptionsWnd->ImageTypeBMP)
      if (FindFirst(path + "\\*.bmp", faAnyFile, sr) == 0)
      {
         do
            names->Add(base + sr.Name);
         while (FindNext(sr) == 0);
         FindClose(sr);
      }
   if (OptionsWnd->ImageTypeJPEG)
      if (FindFirst(path + "\\*.jp*g", faAnyFile, sr) == 0)
      {
         do
            names->Add(base + sr.Name);
         while (FindNext(sr) == 0);
         FindClose(sr);
      }
}
//---------------------------------------------------------------------------
void TMainWnd::SetWallpaper(const char *fname)
{
   TJPEGImage *j;
   TFileStream *s;
   Graphics::TBitmap *b;
   AnsiString bmp;
   AnsiString ext;

   bmp = fname;
   ext = ExtractFileExt(bmp).LowerCase();
   if ((ext == ".jpg") || (ext == ".jpeg"))
   {
      s = new TFileStream(fname, fmOpenRead);
      j = new TJPEGImage();
      b = new Graphics::TBitmap();
      j->LoadFromStream(s);
      b->Assign(j);
      b->SaveToFile(OptionsWnd->TempBitmap);
      delete b;
      delete j;
      delete s;
      bmp = OptionsWnd->TempBitmap;
   }
   SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, bmp.c_str(),
                        SPIF_UPDATEINIFILE + SPIF_SENDCHANGE);
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::TMPauseClick(TObject *Sender)
{
   AnsiString msg;

   TMPause->Checked = !TMPause->Checked;
   if (TMPause->Checked)
   {
      Timer->Enabled = false;
      msg.printf("%s - %i:%02i (paused)", BackgroundFile.c_str(),
                 TimeLeft / 60, TimeLeft % 60);
      if (msg.Length() > 60)
         msg = "..." + msg.SubString((msg.Length() - 60) + 1, 60);
      lstrcpy(IconData.szTip, msg.c_str());
      Shell_NotifyIcon(NIM_MODIFY, &IconData);
      msg.printf("Time remaining: %i:%02i (paused)",
                 TimeLeft / 60, TimeLeft % 60);
      OptionsWnd->Remaining->Caption = msg;
   }
   else
      Timer->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainWnd::TMChangeNowClick(TObject *Sender)
{
   RandomizeBackground();
}
//---------------------------------------------------------------------------
AnsiString TMainWnd::PickWallpaper(TStrings *names)
{
   static int last = -1;
   AnsiString path;

   if ((last == -1) || (OptionsWnd->OrderRandom))
      last = rand();
   else
      last++;
   last %= names->Count;
   BackgroundFile = names->Strings[last];
   path = OptionsWnd->BackgroundPath + "\\" + names->Strings[last];

   return path;
}
//---------------------------------------------------------------------------
void TMainWnd::DoMultiMon(TStringList *names)
{
   // Tips: http://www.realtimesoft.com/multimon/programming/basics.asp
   DEVMODE dm;
   DISPLAY_DEVICE dd;
   DWORD dev;
   int vx, vy, vcx, vcy;
   Graphics::TBitmap *bmp;
   int mwidth, mheight;
   AnsiString path;
   TJPEGImage *j;
   TFileStream *s;
   Graphics::TBitmap *b;
   AnsiString ext;
   int dx, dy;
   TRect rsrc, rdst;
   TSafeRegistry *reg;
   AnsiString tile;

   // We don't use Screen because it's never updated.
   vx = GetSystemMetrics(SM_XVIRTUALSCREEN);
   vy = GetSystemMetrics(SM_YVIRTUALSCREEN);
   vcx = GetSystemMetrics(SM_CXVIRTUALSCREEN);
   vcy = GetSystemMetrics(SM_CYVIRTUALSCREEN);

   bmp = new Graphics::TBitmap();
   bmp->Width = vcx;
   bmp->Height = vcy;
   bmp->Canvas->Brush->Color = GetBackgroundColor();
   bmp->Canvas->FillRect(Rect(0, 0, bmp->Width, bmp->Height));

   dd.cb = sizeof(dd);
   for (dev = 0; EnumDisplayDevices(0, dev, &dd, 0); dev++)
   {
      if (dd.StateFlags & DISPLAY_DEVICE_MIRRORING_DRIVER)
         continue;
      if (!(dd.StateFlags && DISPLAY_DEVICE_ATTACHED_TO_DESKTOP))
         continue;

      ZeroMemory(&dm, sizeof(dm));
      dm.dmSize = sizeof(dm);
      if (!EnumDisplaySettings(dd.DeviceName, ENUM_CURRENT_SETTINGS, &dm))
         EnumDisplaySettings(dd.DeviceName, ENUM_REGISTRY_SETTINGS, &dm);

      mwidth = dm.dmPelsWidth;
      mheight = dm.dmPelsHeight;

      path = PickWallpaper(names);
      ext = ExtractFileExt(path).LowerCase();
      if ((ext == ".jpg") || (ext == ".jpeg"))
      {
         s = new TFileStream(path, fmOpenRead);
         j = new TJPEGImage();
         b = new Graphics::TBitmap();
         j->LoadFromStream(s);
         b->Assign(j);

         if (mwidth >= b->Width)
         {
            rsrc.Left = 0;
            rsrc.Right = b->Width;
            rdst.Left = (mwidth - b->Width) / 2;
            rdst.Right = rdst.Left + b->Width;
         }
         else
         {
            rsrc.Left = (b->Width - mwidth) / 2;
            rsrc.Right = rsrc.Left + mwidth;
            rdst.Left = 0;
            rdst.Right = mwidth;
         }
         if (mheight >= b->Height)
         {
            rsrc.Top = 0;
            rsrc.Bottom = b->Height;
            rdst.Top = (mheight - b->Height) / 2;
            rdst.Bottom = rdst.Top + b->Height;
         }
         else
         {
            rsrc.Top = (b->Height - mheight) / 2;
            rsrc.Bottom = rsrc.Top + mheight;
            rdst.Top = 0;
            rdst.Bottom = mheight;
         }
         dx = dm.dmPosition.x - vx;
         dy = dm.dmPosition.y - vy;
         rdst.Left += dx;
         rdst.Right += dx;
         rdst.Top += dy;
         rdst.Bottom += dy;

         bmp->Canvas->CopyRect(rdst, b->Canvas, rsrc);
         delete b;
         delete j;
         delete s;
      }
   }

   bmp->SaveToFile(OptionsWnd->TempBitmap);
   delete bmp;

   reg = new TSafeRegistry();
   reg->RootKey = HKEY_CURRENT_USER;
   reg->OpenKey("Control Panel\\Desktop", false);
   tile = reg->SReadString("TileWallpaper");
   reg->SWriteString("TileWallpaper", "1");
   reg->CloseKey();
   if (tile == "0")
   {
      reg->OpenKey("Software\\ACZ\\Random Background", true);
      reg->SWriteBool("ResetTile", true);
      reg->CloseKey();
   }
   delete reg;

   SystemParametersInfo(SPI_SETDESKWALLPAPER, 0,
                        OptionsWnd->TempBitmap.c_str(),
                        SPIF_UPDATEINIFILE + SPIF_SENDCHANGE);
}
//---------------------------------------------------------------------------
TColor TMainWnd::GetBackgroundColor()
{
   TSafeRegistry *reg;
   AnsiString color;
   int r, g, b;

   reg = new TSafeRegistry();
   reg->RootKey = HKEY_CURRENT_USER;
   reg->OpenKey("Control Panel\\Colors", false);
   color = reg->SReadString("Background", "0 0 0");
   reg->CloseKey();
   delete reg;

   r = 0;
   g = 0;
   b = 0;
   sscanf(color.c_str(), "%i %i %i", &r, &g, &b);
   r &= 0xFF;
   g &= 0xFF;
   b &= 0xFF;

   return (TColor) ((b << 16) | (g << 8) | r);
}
//---------------------------------------------------------------------------
void TMainWnd::ResetTile()
{
   TSafeRegistry *reg;
   bool reset;

   reg = new TSafeRegistry();
   reg->RootKey = HKEY_CURRENT_USER;
   reg->OpenKey("Software\\ACZ\\Random Background", true);
   reset = reg->SReadBool("ResetTile", false);
   reg->DeleteValue("ResetTile");
   reg->CloseKey();
   if (reset)
   {
      reg->OpenKey("Control Panel\\Desktop", false);
      reg->SWriteString("TileWallpaper", "0");
      reg->CloseKey();
   }
   delete reg;
}
//---------------------------------------------------------------------------
