//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <vcl/FileCtrl.hpp>
#include <vcl/registry.hpp>
#include <stdlib.h>

#include "Options.h"
#include "SafeRegistry.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TOptionsWnd *OptionsWnd;
//---------------------------------------------------------------------------
__fastcall TOptionsWnd::TOptionsWnd(TComponent* Owner)
   : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TOptionsWnd::FormShow(TObject *Sender)
{
   SetForegroundWindow(Handle);
   LoadSettings();
   ValidateSettings();
   Interval->Text = AnsiString(TimeInterval);
   TemporaryBitmap->Text = TempBitmap;
   BGPath->Text = BackgroundPath;
   BGRDirs->Checked = RecurseDirs;
   ITBMP->Checked = ImageTypeBMP;
   ITJPEG->Checked = ImageTypeJPEG;
   DORandom->Checked = OrderRandom;
   DOCycled->Checked = !OrderRandom;
   OMultiMon->Checked = MultiMon;
   ButtonOK->SetFocus();
}
//---------------------------------------------------------------------------
void __fastcall TOptionsWnd::BGPathBrowseClick(TObject *Sender)
{
   AnsiString dir;

   dir = BGPath->Text;
   if (SelectDirectory(AnsiString("Select background path"), "", dir))
      BGPath->Text = dir;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsWnd::ButtonOKClick(TObject *Sender)
{
   TSafeRegistry *reg;

   ValidateSettings();
   TimeInterval = atoi(Interval->Text.c_str());
   TempBitmap = TemporaryBitmap->Text;
   BackgroundPath = BGPath->Text;
   RecurseDirs = BGRDirs->Checked;
   ImageTypeBMP = ITBMP->Checked;
   ImageTypeJPEG = ITJPEG->Checked;
   OrderRandom = DORandom->Checked;
   MultiMon = OMultiMon->Checked;
   reg = new TSafeRegistry();
   reg->RootKey = HKEY_CURRENT_USER;
   reg->OpenKey("Software\\ACZ\\Random Background", true);
   reg->SWriteInteger("Interval", TimeInterval);
   reg->SWriteString("TempBitmap", TempBitmap);
   reg->SWriteString("BackgroundPath", BackgroundPath);
   reg->SWriteBool("RecurseDirs", RecurseDirs);
   reg->SWriteBool("ImageTypeBMP", ImageTypeBMP);
   reg->SWriteBool("ImageTypeJPEG", ImageTypeJPEG);
   reg->SWriteBool("OrderRandom", OrderRandom);
   reg->SWriteBool("MultiMon", MultiMon);
   reg->CloseKey();
   delete reg;
}
//---------------------------------------------------------------------------
void TOptionsWnd::ValidateSettings()
{
   int i;

   i = atoi(Interval->Text.c_str());
   if ((i < 1) || (i > 1440))
      Interval->Text = AnsiString(TimeInterval);
   if ((!ITBMP->Checked) && (!ITJPEG->Checked))
   {
      ITBMP->Checked = true;
      ITJPEG->Checked = true;
   }
}
//---------------------------------------------------------------------------
void TOptionsWnd::LoadSettings()
{
   TSafeRegistry *reg;

   TimeInterval = 10;
   TempBitmap = "C:\\wallpaper.bmp";
   BackgroundPath = GetCurrentDir();
   RecurseDirs = true;
   ImageTypeBMP = true;
   ImageTypeJPEG = true;
   OrderRandom = true;
   MultiMon = false;
   reg = new TSafeRegistry();
   reg->RootKey = HKEY_CURRENT_USER;
   reg->OpenKey("Software\\ACZ\\Random Background", true);
   TimeInterval = reg->SReadInteger("Interval", TimeInterval);
   TempBitmap = reg->SReadString("TempBitmap", TempBitmap);
   BackgroundPath = reg->SReadString("BackgroundPath", BackgroundPath);
   RecurseDirs = reg->SReadBool("RecurseDirs", RecurseDirs);
   ImageTypeBMP = reg->SReadBool("ImageTypeBMP", ImageTypeBMP);
   ImageTypeJPEG = reg->SReadBool("ImageTypeJPEG", ImageTypeJPEG);
   OrderRandom = reg->SReadBool("OrderRandom", OrderRandom);
   MultiMon = reg->SReadBool("MultiMon", MultiMon);
   reg->CloseKey();
   delete reg;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsWnd::FormCreate(TObject *Sender)
{
   LoadSettings();
   ValidateSettings();
}
//---------------------------------------------------------------------------
void __fastcall TOptionsWnd::TBPathBrowserClick(TObject *Sender)
{
   TBSaveDlg->FileName = TemporaryBitmap->Text;
   if (TBSaveDlg->Execute())
      TemporaryBitmap->Text = TBSaveDlg->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsWnd::LaunchBrowser(const char *url)
{
   const char NAME[] = "AllowWindowReuse";
   TSafeRegistry *reg;
   bool key;
   bool reuse;

   reg = new TSafeRegistry();
   reg->LazyWrite = false;
   reg->RootKey = HKEY_CURRENT_USER;
   key = reg->OpenKey("Software\\Microsoft\\Internet Explorer\\Main", false);
   if (key)
   {
      reuse = reg->SReadBool(NAME);
      reg->SWriteBool(NAME, 0);
   }
   ShellExecute(NULL, "open", url, NULL, NULL, SW_MAXIMIZE);
   if (key)
      reg->SWriteBool(NAME, reuse);
   delete reg;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsWnd::LWebsiteClick(TObject *Sender)
{
   LaunchBrowser("http://david.acz.org/randombg/");
}
//---------------------------------------------------------------------------
